Type: Package
Package: MDP2
Title: Markov Decision Processes (MDPs)
Version: 2.2.2.0
Authors@R: c(
    person("Lars", "Relund Nielsen", , "lars@relund.dk", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-4802-3071")))
Author: Lars Relund Nielsen [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4802-3071>)
Maintainer: Lars Relund Nielsen <lars@relund.dk>
Description: Create and optimize (semi) MDPs with discrete time steps and
    state space. Both hierarchical and ordinary-traditional MDPs can be
    modeled.
License: GPL (>= 3)
URL: https://relund.github.io/mdp/, https://github.com/relund/mdp/,
        http://relund.github.io/mdp/
BugReports: https://github.com/relund/mdp/issues
Depends: R (>= 4.1.0)
Imports: diagram, dplyr, stringr, tidyr, magrittr, methods, purrr,
        rlang, tibble, Rcpp (>= 0.11.5)
Suggests: knitr, Matrix, rmarkdown, testthat (>= 3.0.0), readr, xml2,
        covr, roxygen2
LinkingTo: Rcpp, RcppArmadillo
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2026-06-04 09:51:52 UTC; au15463
Repository: CRAN
Date/Publication: 2026-06-12 19:30:14 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-06-12 20:15:05 UTC; unix
Archs: MDP2.so.dSYM
