% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mdp.R
\name{getWIdx}
\alias{getWIdx}
\title{Return the index of a weight in the model. Note that index always start from zero (C++ style), i.e. the first weight, the first state at a stage etc has index 0.}
\usage{
getWIdx(mdp, wLbl)
}
\arguments{
\item{mdp}{The MDP loaded using \code{\link[=loadMDP]{loadMDP()}}.}

\item{wLbl}{The label/string of the weight.}
}
\value{
The index (integer).
}
\description{
Return the index of a weight in the model. Note that index always start from zero (C++ style), i.e. the first weight, the first state at a stage etc has index 0.
}
