% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mdp.R
\name{saveMDP}
\alias{saveMDP}
\title{Save the MDP to binary files}
\usage{
saveMDP(mdp, prefix = "", getLog = TRUE)
}
\arguments{
\item{mdp}{The MDP loaded using \code{\link[=loadMDP]{loadMDP()}}.}

\item{prefix}{A character string with the prefix added to \code{binNames}. Used to identify a specific model.}

\item{getLog}{Output the log as a message.}
}
\value{
???
}
\description{
Currently do not save external files.
}
